@@ -13,10 +13,12 @@ from .api import Fitbit, FitbitConsumer, FitbitOauthClient
 
 __title__ = 'fitbit'
 __author__ = 'Issac Kelly and ORCAS'
-__author_email__ = 'pperez@orcasinc.com'
+__author_email__ = 'bpitcher@orcasinc.com'
 __copyright__ = 'Copyright 2012 ORCAS'
 __license__ = 'Apache 2.0'
 
-__version__ = '0.0.2'
+__version__ = '0.0.5'
 
 # Module namespace.
+
+all_tests = []
