@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 import oauth2 as oauth
 import requests
-import urlparse
 import json
 import datetime
 import urllib
 
+from requests_oauthlib import OAuth1Session
 from fitbit.exceptions import (BadResponse, DeleteError, HTTPBadRequest,
                                HTTPUnauthorized, HTTPForbidden,
                                HTTPServerError, HTTPConflict, HTTPNotFound)
@@ -53,11 +53,13 @@ class FitbitOauthClient(oauth.Client):
         """
         if not method:
             method = 'POST' if data else 'GET'
+        headers = kwargs.pop('headers', {})
         request = oauth.Request.from_consumer_and_token(self._consumer, self._token, http_method=method, http_url=url, parameters=data)
         request.sign_request(self._signature_method, self._consumer,
                              self._token)
+        headers.update(request.to_header())
         response = self._request(method, url, data=data,
-                                 headers=request.to_header())
+                                 headers=headers)
 
         if response.status_code == 401:
             raise HTTPUnauthorized(response)
@@ -133,16 +135,18 @@ class FitbitOauthClient(oauth.Client):
         from that and save them, then pass them as user_key and user_secret in future
         API calls to fitbit to get this user's data.
         """
-        request = oauth.Request.from_consumer_and_token(self._consumer, token, http_method='POST', http_url=self.access_token_url, parameters={'oauth_verifier': verifier})
-        body = "oauth_verifier=%s" % verifier
-        response = self._request('POST', self.access_token_url, data=body,
-                                 headers=request.to_header())
-        if response.status_code != 200:
-            # TODO custom exceptions
-            raise Exception("Invalid response %s." % response.content)
-        params = urlparse.parse_qs(response.content, keep_blank_values=False)
-        self.user_id = params['encoded_user_id'][0]
-        self._token = oauth.Token.from_string(response.content)
+        client = OAuth1Session(
+            self._consumer.key,
+            client_secret=self._consumer.secret,
+            resource_owner_key=token.key,
+            resource_owner_secret=token.secret,
+            verifier=verifier)
+        response = client.fetch_access_token(self.access_token_url)
+
+        self.user_id = response['encoded_user_id']
+        self._token = oauth.Token(
+            key=response['oauth_token'],
+            secret=response['oauth_token_secret'])
         return self._token
 
 
@@ -152,6 +156,7 @@ class Fitbit(object):
 
     API_ENDPOINT = "https://api.fitbit.com"
     API_VERSION = 1
+    WEEK_DAYS = ['SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY']
 
     _resource_list = [
         'body',
@@ -432,6 +437,15 @@ class Fitbit(object):
         )
         return self.make_request(url, method='POST')
 
+    def log_activity(self, data):
+        """
+        https://wiki.fitbit.com/display/API/API-Log-Activity
+        """
+        url = "%s/%s/user/-/activities.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION)
+        return self.make_request(url, data = data)
+
     def delete_favorite_activity(self, activity_id):
         """
         https://wiki.fitbit.com/display/API/API-Delete-Favorite-Activity
@@ -495,6 +509,130 @@ class Fitbit(object):
         )
         return self.make_request(url)
 
+    def get_alarms(self, device_id):
+        """
+        https://wiki.fitbit.com/display/API/API-Devices-Get-Alarms
+        """
+        url = "%s/%s/user/-/devices/tracker/%s/alarms.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            device_id
+        )
+        return self.make_request(url)
+
+    def add_alarm(self, device_id, alarm_time, week_days, recurring=False, enabled=True, label=None,
+                     snooze_length=None, snooze_count=None, vibe='DEFAULT'):
+        """
+        https://wiki.fitbit.com/display/API/API-Devices-Add-Alarm
+        alarm_time should be a timezone aware datetime object.
+        """
+        url = "%s/%s/user/-/devices/tracker/%s/alarms.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            device_id
+        )
+        alarm_time = alarm_time.strftime("%H:%M%z")
+        # Check week_days list
+        if not isinstance(week_days, list):
+            raise ValueError("Week days needs to be a list")
+        for day in week_days:
+            if day not in self.WEEK_DAYS:
+                raise ValueError("Incorrect week day %s. see WEEK_DAY_LIST." % day)
+        data = {
+            'time': alarm_time,
+            'weekDays': week_days,
+            'recurring': recurring,
+            'enabled': enabled,
+            'vibe': vibe
+        }
+        if label:
+            data['label'] = label
+        if snooze_length:
+            data['snoozeLength'] = snooze_length
+        if snooze_count:
+            data['snoozeCount'] = snooze_count
+        return self.make_request(url, data=data, method="POST")
+        # return
+
+    def update_alarm(self, device_id, alarm_id, alarm_time, week_days, recurring=False, enabled=True, label=None,
+                     snooze_length=None, snooze_count=None, vibe='DEFAULT'):
+        """
+        https://wiki.fitbit.com/display/API/API-Devices-Update-Alarm
+        alarm_time should be a timezone aware datetime object.
+        """
+        # TODO Refactor with create_alarm. Tons of overlap.
+        # Check week_days list
+        if not isinstance(week_days, list):
+            raise ValueError("Week days needs to be a list")
+        for day in week_days:
+            if day not in self.WEEK_DAYS:
+                raise ValueError("Incorrect week day %s. see WEEK_DAY_LIST." % day)
+        url = "%s/%s/user/-/devices/tracker/%s/alarms/%s.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            device_id,
+            alarm_id
+        )
+        alarm_time = alarm_time.strftime("%H:%M%z")
+
+        data = {
+            'time': alarm_time,
+            'weekDays': week_days,
+            'recurring': recurring,
+            'enabled': enabled,
+            'vibe': vibe
+        }
+        if label:
+            data['label'] = label
+        if snooze_length:
+            data['snoozeLength'] = snooze_length
+        if snooze_count:
+            data['snoozeCount'] = snooze_count
+        return self.make_request(url, data=data, method="POST")
+        # return
+
+    def delete_alarm(self, device_id, alarm_id):
+        """
+        https://wiki.fitbit.com/display/API/API-Devices-Delete-Alarm
+        """
+        url = "%s/%s/user/-/devices/tracker/%s/alarms/%s.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            device_id,
+            alarm_id
+        )
+        return self.make_request(url, method="DELETE")
+
+    def get_sleep(self, date):
+        """
+        https://wiki.fitbit.com/display/API/API-Get-Sleep
+        date should be a datetime.date object.
+        """
+        url = "%s/%s/user/-/sleep/date/%s-%s-%s.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            date.year,
+            date.month,
+            date.day
+        )
+        return self.make_request(url)
+
+    def log_sleep(self, start_time, duration):
+        """
+        https://wiki.fitbit.com/display/API/API-Log-Sleep
+        start time should be a datetime object. We will be using the year, month, day, hour, and minute.
+        """
+        data = {
+            'startTime': start_time.strftime("%H:%M"),
+            'duration': duration,
+            'date': start_time.strftime("%Y-%m-%d"),
+        }
+        url = "%s/%s/user/-/sleep" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+        )
+        return self.make_request(url, data=data, method="POST")
+
     def activities_list(self):
         """
         https://wiki.fitbit.com/display/API/API-Browse-Activities
@@ -548,6 +686,118 @@ class Fitbit(object):
         )
         return self.make_request(url)
 
+    def get_bodyweight(self, base_date=None, user_id=None, period=None, end_date=None):
+        """
+        https://wiki.fitbit.com/display/API/API-Get-Body-Weight
+        base_date should be a datetime.date object (defaults to today),
+        period can be '1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max' or None
+        end_date should be a datetime.date object, or None.
+
+        You can specify period or end_date, or neither, but not both.
+        """
+        if not base_date:
+            base_date = datetime.date.today()
+
+        if not user_id:
+            user_id = '-'
+
+        if period and end_date:
+            raise TypeError("Either end_date or period can be specified, not both")
+
+        if not isinstance(base_date, basestring):
+            base_date_string = base_date.strftime('%Y-%m-%d')
+        else:
+            base_date_string = base_date
+
+        if period:
+            if not period in ['1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max']:
+                raise ValueError("Period must be one of '1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max'")
+
+            url = "%s/%s/user/%s/body/log/weight/date/%s/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+                period
+            )
+        elif end_date:
+            if not isinstance(end_date, basestring):
+                end_string = end_date.strftime('%Y-%m-%d')
+            else:
+                end_string = end_date
+
+            url = "%s/%s/user/%s/body/log/weight/date/%s/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+                end_string
+            )
+        else:
+            url = "%s/%s/user/%s/body/log/weight/date/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+            )
+        return self.make_request(url)
+
+    def get_bodyfat(self, base_date=None, user_id=None, period=None, end_date=None):
+        """
+        https://wiki.fitbit.com/display/API/API-Get-Body-fat
+        base_date should be a datetime.date object (defaults to today),
+        period can be '1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max' or None
+        end_date should be a datetime.date object, or None.
+
+        You can specify period or end_date, or neither, but not both.
+        """
+        if not base_date:
+            base_date = datetime.date.today()
+
+        if not user_id:
+            user_id = '-'
+
+        if period and end_date:
+            raise TypeError("Either end_date or period can be specified, not both")
+
+        if not isinstance(base_date, basestring):
+            base_date_string = base_date.strftime('%Y-%m-%d')
+        else:
+            base_date_string = base_date
+
+        if period:
+            if not period in ['1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max']:
+                raise ValueError("Period must be one of '1d', '7d', '30d', '1w', '1m', '3m', '6m', '1y', 'max'")
+
+            url = "%s/%s/user/%s/body/log/fat/date/%s/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+                period
+            )
+        elif end_date:
+            if not isinstance(end_date, basestring):
+                end_string = end_date.strftime('%Y-%m-%d')
+            else:
+                end_string = end_date
+
+            url = "%s/%s/user/%s/body/log/fat/date/%s/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+                end_string
+            )
+        else:
+            url = "%s/%s/user/%s/body/log/fat/date/%s.json" % (
+                self.API_ENDPOINT,
+                self.API_VERSION,
+                user_id,
+                base_date_string,
+            )
+        return self.make_request(url)
+
     def get_friends(self, user_id=None):
         """
         https://wiki.fitbit.com/display/API/API-Get-Friends
@@ -622,6 +872,19 @@ class Fitbit(object):
         """
         return self.respond_to_invite(other_user_id, accept=False)
 
+    def get_badges(self, user_id=None):
+        """
+        https://wiki.fitbit.com/display/API/API-Get-Badges
+        """
+        if not user_id:
+            user_id = '-'
+        url = "%s/%s/user/%s/badges.json" % (
+            self.API_ENDPOINT,
+            self.API_VERSION,
+            user_id
+        )
+        return self.make_request(url)
+
     def subscription(self, subscription_id, subscriber_id, collection=None,
                      method='POST'):
         """
